﻿using System;
using System.Windows.Forms;
using SolutionManager.IO;

namespace SolutionManager.Business.UI.Froms
{
	public partial class NewFolderForm : Form, IDirectorySettings
	{
		#region Fields

		private bool _canceled = false;
		private TreeNode _treeNode = null;
		private SearchDirectory _directory = null;

		#endregion

		#region Constructors

		public NewFolderForm()
		{
			InitializeComponent();
		}

		public NewFolderForm(TreeNode treeNode) : this()
		{
			_treeNode = treeNode;
		}

		#endregion

		#region Properties

		#region INewFolderForm Members

		public bool SearchChildren
		{
			get
			{
				return chkSearchChildren.Checked;
			}
			set
			{
				chkSearchChildren.Checked = value;
			}
		}

		public bool AddSearch
		{
			get
			{
				return chkAddSearch.Checked;
			}
			set
			{
				chkAddSearch.Checked = value;
				chkSearchChildren.Enabled = value;
				label2.Enabled = value;
				txtPath.Enabled = value;
				btnSelectFolder.Enabled = value;
			}
		}

		public string Path
		{
			get
			{
				return txtPath.Text;
			}
			set
			{
				if (!string.IsNullOrEmpty(value))
				{
					txtPath.Text = value;
				}
			}
		}

		public string FriendlyName
		{
			get
			{
				return txtName.Text;
			}
			set
			{
				txtName.Text = value;
			}
		}

		public TreeNode ParentTreeNode
		{
			get
			{
				return _treeNode;
			}
			set
			{
				_treeNode = value;
			}
		}

		public SearchDirectory Directory
		{
			get
			{
				return _directory;
			}
		}

		#endregion

		#endregion

		#region Methods

		#region Events

		private void NewFolderForm_Load(object sender, EventArgs e)
		{
			AddSearch = false;
		}

		private void btnSelectFolder_Click(object sender, EventArgs e)
		{
			FolderBrowserDialog folderBrowser = new FolderBrowserDialog();
			folderBrowser.ShowNewFolderButton = false;

			if (folderBrowser.ShowDialog() == DialogResult.OK)
			{
				Path = folderBrowser.SelectedPath;
				_directory = new SearchDirectory(txtName.Text, Path);
			}
		}

		private void btnOk_Click(object sender, EventArgs e)
		{
			if (!txtName.Text.Equals("Solution Manager"))
			{
				Close();
			}
			else
			{
				// HACK: Must allow rename of rootNode name="Solution Manager".
				MessageBox.Show("Folder name must not be \"Solution Manager\"");
			}
		}

		private void btnCancel_Click(object sender, EventArgs e)
		{
			_canceled = true;
		}

		private void chkAddSearch_CheckedChanged(object sender, EventArgs e)
		{
			AddSearch = chkAddSearch.Checked;
		}

		#endregion

		public IDirectorySettings NewFolderShowDialog(bool addDirectoryEnabled)
		{
			IDirectorySettings settings = null;
			chkAddSearch.Enabled = addDirectoryEnabled;

			ShowDialog();

			if (!_canceled)
			{
				settings = this;
			}

			return settings;
		}

		#endregion
	}
}
