﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using SolutionManager.IO;
using SolutionManager.IO.Xml;
using SolutionManager.Business.UI.Froms;
using System.Windows.Forms;

namespace SolutionManager.Business.UI
{
	public class SolutionControlControler
	{
		#region Fields

		private System.Windows.Forms.TreeNode _rootNode = null;
		private IDirectoryControlView _directoryControlView = null;
		private ISolutionControlView _solutionControlView = null;
		private ISolutionManagerSettings _solutionManagerSettings = null;
		private SolutionControlModel _solutionControlModel = null;

		#endregion

		#region Constructors

		public SolutionControlControler()
		{
			SearchDirectory rootDirectory = new SearchDirectory("Solution Manager", "", "");
			_solutionControlModel = new SolutionControlModel(rootDirectory);

			_rootNode = new System.Windows.Forms.TreeNode("Solution Manager", 2, 2);
			_rootNode.Tag = rootDirectory;
		}

		public SolutionControlControler(IDirectoryControlView directoryControlView, ISolutionControlView solutionControlModel, ISolutionManagerSettings solutionManagerSettings)
			: this()
		{
			_directoryControlView = directoryControlView;
			_solutionControlView = solutionControlModel;
			_solutionManagerSettings = solutionManagerSettings;

			DisplayMessageInSolutionsView("Searching...");
			System.Threading.ThreadPool.QueueUserWorkItem(ReadSolutionManagerFile, null);
		}

		#endregion

		#region Methods

		/// <summary>
		/// Reads the SolutionManager.xml and binds the ListViews.
		/// </summary>
		/// <param name="obj">Not currently used and can be null.</param>
		private void ReadSolutionManagerFile(object obj)
		{
			_solutionControlModel.Initialize();

			if (_solutionControlModel.IsInitialized)
			{
				OnThreadCompleted_BindListView();
			}
		}

		private void RemoveDirectoryRecursive(SearchDirectory directoryToRemove, List<SearchDirectory> directories)
		{
			if (directoryToRemove != null)
			{
				if (!directories.Contains(directoryToRemove))
				{
					int directoryCount = directories.Count;

					for (int i = 0; i < directories.Count; i++)
					{
						RemoveDirectoryRecursive(directoryToRemove, directories[i].ChildDirectories);
					}
				}
				else
				{
					directories.Remove(directoryToRemove);
					directoryToRemove = null;
				}
			}
		}


		internal protected void BindDirectoriesAndSolutions(int directoryIndex)
		{
			SearchDirectory directory = _solutionControlModel.SearchDirectory;

			if (directory != null)
			{
				int directoryCount = directory.ChildDirectories.Count;

				if (directoryCount > 0)
				{
					System.Windows.Forms.TreeNode parentNode = null;
					System.Windows.Forms.TreeNode childNodes = null;
					_rootNode.Nodes.Clear();

					for (int i = 0; i < directoryCount; i++)
					{
						// If it makes it through this if statement then it is
						// either not excluded or we are including all files.
						if ((!directory.ChildDirectories[i].Excluded) || (_solutionManagerSettings.ShowExcludedFiles))
						{
							parentNode = new System.Windows.Forms.TreeNode(directory.ChildDirectories[i].Name, 1, 0);
							parentNode.Tag = directory.ChildDirectories[i];
							parentNode.ToolTipText = directory.ChildDirectories[i].Directory;

							if (directory.ChildDirectories[i].Excluded)
							{
								parentNode.ForeColor = System.Drawing.Color.Gray;
								System.Drawing.Font font = new System.Drawing.Font("Arial", 8.0F, System.Drawing.FontStyle.Italic);
								parentNode.NodeFont = font;
							}

							if (directory.ChildDirectories[i].HasChildrenDirectories)
							{
								int childDirectoryCount = directory.ChildDirectories[i].ChildDirectories.Count;

								if (childDirectoryCount > 0)
								{
									for (int n = 0; n < childDirectoryCount; n++)
									{
										childNodes = AddChildDirectories(directory.ChildDirectories[i].ChildDirectories[n]);

										if (childNodes != null)
										{
											parentNode.Nodes.Add(childNodes);
										}
									}
								}
							}

							_rootNode.Nodes.Add(parentNode);
						}
					}

					if (directoryIndex < directoryCount)
					{
						BindSolutions(directory.ChildDirectories[directoryIndex].Solutions);
						directory = null;
					}
				}
				else
				{
					_directoryControlView.DirectoryTreeViewNodes.Clear();
					DisplayMessageInSolutionsView("No results to display.");
				}

				_directoryControlView.DirectoryTreeViewNodes.Clear();
				_rootNode.ImageIndex = 2;
				_rootNode.SelectedImageIndex = 2;
				_directoryControlView.DirectoryTreeViewNodes.Add(_rootNode);
				_directoryControlView.DirectoryTreeViewNodes[0].Expand();
			}
		}

		internal protected void BindSolutions(List<Solution> solutions)
		{
			if (solutions != null)
			{
				int solutionCount = solutions.Count;

				if (solutionCount > 0)
				{
					// Create a ListViewItem to add to the ListView.
					ListViewItem item = null;
					int counter = 0;
					_solutionControlView.SolutionsListView.Enabled = true;
					_solutionControlView.SolutionsListView.BeginUpdate();
					_solutionControlView.SolutionListViewItems.Clear();

					for (int i = 0; i < solutionCount; i++)
					{
						// If it makes it through this if statement then it is
						// either not excluded or we are including all files.
						if ((!solutions[i].Excluded) || (_solutionManagerSettings.ShowExcludedFiles))
						{
							item = new System.Windows.Forms.ListViewItem(solutions[i].Name, 2);
							item.Tag = solutions[i];

							// If the current Solution is excluded or they are
							// showing excluded files change the font color and image.
							if (solutions[i].Excluded)
							{
								item.ForeColor = System.Drawing.Color.Gray;
								item.ImageIndex = 3;
								System.Drawing.Font font = new System.Drawing.Font("Arial", 8.0F, System.Drawing.FontStyle.Italic);
								item.Font = font;
							}

							// Add the ListViewItem and SubItems to the ListView.
							_solutionControlView.SolutionListViewItems.Add(item);
							_solutionControlView.SolutionListViewItems[counter].SubItems.Add(solutions[i].FullName);
							_solutionControlView.SolutionListViewItems[counter].SubItems.Add(solutions[i].CreationTime.ToShortDateString());
							
							// Increment the ListViewItem counter.
							counter++;
						}
					}

					_solutionControlView.SolutionsListView.EndUpdate();
				}
				else
				{
					// No results found so notify the user.
					DisplayMessageInSolutionsView("No results to display.");
				}
			}
		}

		/// <summary>
		/// Creates a Directory node to be bound to the treeview.
		/// </summary>
		/// <param name="directory">The directory to create the node from.</param>
		/// <returns>System.Windows.Forms.TreeNode</returns>
		internal TreeNode AddChildDirectories(SearchDirectory directory)
		{
			System.Windows.Forms.TreeNode treeNode = null;

			if (directory != null)
			{
				// If it makes it through in this if statement then the directory
				// is either included or they are including all the excluded files.
				if ((!directory.Excluded) || (_solutionManagerSettings.ShowExcludedFiles))
				{
					treeNode = new System.Windows.Forms.TreeNode(directory.Name, 1, 0);
					treeNode.Tag = directory;
					treeNode.ToolTipText = directory.Directory;

					if (directory.Excluded)
					{
						treeNode.ForeColor = System.Drawing.Color.Gray;
						System.Drawing.Font font = new System.Drawing.Font("Arial", 8.0F, System.Drawing.FontStyle.Italic);
						treeNode.NodeFont = font;
					}

					int directoryCount = directory.ChildDirectories.Count;

					if (directoryCount > 0)
					{
						System.Windows.Forms.TreeNode childDirectoryNode = null;

						for (int i = 0; i < directoryCount; i++)
						{
							childDirectoryNode = AddChildDirectories(directory.ChildDirectories[i]);

							if (childDirectoryNode != null)
							{
								treeNode.Nodes.Add(childDirectoryNode);
							}
						}
					}
				}
			}

			return treeNode;
		}

		internal void DisplayMessageInSolutionsView(string listMessage)
		{
			if (!string.IsNullOrEmpty(listMessage))
			{
				_solutionControlView.SolutionListViewItems.Clear();

				System.Windows.Forms.ListViewItem item = new System.Windows.Forms.ListViewItem(string.Empty);
				item.SubItems.Add(listMessage);
				item.ForeColor = System.Drawing.Color.Gray;

				_solutionControlView.SolutionListViewItems.Add(item);

				item = null;

				_solutionControlView.SolutionsListView.Enabled = false;
			}
		}

		internal void AddVirtualNode(IDirectorySettings directorySettings)
		{
			int index = 0;

			if (directorySettings.ParentTreeNode != null)
			{
				index = directorySettings.ParentTreeNode.Index;
			}

			_solutionControlModel.AddVirtualDirectory(directorySettings);
			BindDirectoriesAndSolutions(index);
		}

		/// <summary>
		/// Binds the SolutionsListView and DirectoryTreeView after the thread has completed.
		/// </summary>
		internal void OnThreadCompleted_BindListView()
		{
			if (_solutionControlView.SolutionsListView.InvokeRequired)
			{
				SolutionManager.Business.Events.ThreadCompletedEventHandler threadCompleted = new SolutionManager.Business.Events.ThreadCompletedEventHandler(OnThreadCompleted_BindListView);
				_solutionControlView.SolutionControlViewInvoke(threadCompleted);
			}
			else
			{
				BindDirectoriesAndSolutions(0);
			}
		}

		internal void RemoveDirectoryNode(TreeNode treeNode, string parentNode)
		{
			if ((treeNode != null) && (!string.IsNullOrEmpty(parentNode)))
			{
				SearchDirectory directory = treeNode.Tag as SearchDirectory;

				if (directory != null)
				{
					List<SearchDirectory> directories = _solutionControlModel.SearchDirectory.ChildDirectories;
					RemoveDirectoryRecursive(directory, directories);

					treeNode.Parent.Nodes.Remove(treeNode);

					_solutionControlModel.SaveSolutionSettings();
					BindDirectoriesAndSolutions(0);
				}
			}
		}

		internal void ExcludeSolutionNode(ListViewItem item, bool bindSolutions)
		{
			if (item != null)
			{
				Solution solution = item.Tag as Solution;

				if (solution != null)
				{
					solution.Excluded = !solution.Excluded;

					if (solution.Parent != null)
					{
						if (solution.Parent != null)
						{
							_solutionControlModel.SaveSolutionSettings();

							if (bindSolutions)
							{
								BindSolutions(solution.Parent.Solutions);
							}
						}
					}
				}
			}
		}

		internal void MoveDirectoryNode(TreeNode nodeToMove, TreeNode parentNode, TreeNode nodeToAddTo)
		{
			SearchDirectory directoryToAddTo = nodeToAddTo.Tag as SearchDirectory;

			if (directoryToAddTo != null)
			{
				SearchDirectory directory = nodeToMove.Tag as SearchDirectory;
				SearchDirectory parentDirectory = parentNode.Tag as SearchDirectory;

				if ((directory != null) && (parentDirectory != null))
				{
					directory.ParentName = directoryToAddTo.Name;
					directoryToAddTo.ChildDirectories.Add(directory);
					parentDirectory.ChildDirectories.Remove(directory);
					_solutionControlModel.SaveSolutionSettings(_directoryControlView.DirectoryTreeView.Nodes[0].Tag as SearchDirectory);
					//_solutionControlModel.MoveDirectory(directoryToAddTo, directory);
				}
			}
		}

		internal void RenameSearchDirectory(string id, string newName)
		{
			_solutionControlModel.RenameDirectory(id, newName);
		}

		internal void SaveSettings(bool windowOpen)
		{
			_solutionControlModel.SaveAddinSettings(windowOpen);
		}


		/// <summary>
		/// Adds a SolutionManager.IO.SearchDirectory to the directory list.
		/// This should be the start of a thread.
		/// </summary>
		/// <param name="searchDirectory">The root directory to search solutions for.</param>
		public void AddSearchDirectory(object newFolderSettings)
		{
			IDirectorySettings directorySettings = newFolderSettings as IDirectorySettings;

			if (directorySettings != null)
			{
				_solutionControlModel.AddSearchDirectory(directorySettings);
				OnThreadCompleted_BindListView();
			}
		}

		#endregion
	}
}