﻿using System;
using System.Windows.Forms;
using EnvDTE80;
using SolutionManager.IO;
using SolutionManager.Business.UI.Froms;

namespace SolutionManager.Business.UI
{
	public partial class SolutionControlView : UserControl, ISolutionControlView, IDirectoryControlView, ISolutionManagerSettings
	{
		#region Fields

		private const string _SolutionManagerTitle = "Solution Manager {0} {1} {0} {2}";

		private bool _allDirectoryTreeNodesExpanded = false;
		private DTE2 _applicationObject = null;
		private EnvDTE.Window _window = null;
		private SolutionControlControler _solutionManagerControler = null;

		#endregion

		#region Constructors

		public SolutionControlView()
		{
			InitializeComponent();
		}

		#endregion

		#region Properties

		#region ISolutionControlView Members

		public bool LstSolutionsInvokeRequired
		{
			get
			{
				return lstSolutions.InvokeRequired;
			}
		}

		public ListView.ListViewItemCollection SolutionListViewItems
		{
			get
			{
				return lstSolutions.Items;
			}
		}

		public ListView SolutionsListView
		{
			get
			{
				return lstSolutions;
			}
		}

		#endregion

		#region IDirectoryControlView Members

		public TreeView DirectoryTreeView
		{
			get
			{
				return directoryTreeView;
			}
		}

		public TreeNodeCollection DirectoryTreeViewNodes
		{
			get
			{
				return directoryTreeView.Nodes;
			}
		}

		#endregion

		#region ISolutionManagerSettings Members

		public bool ShowExcludedFiles
		{
			get
			{
				return showExcludedSolutionsToolStripMenuItem.Checked;
			}
			set
			{
				System.Collections.Generic.List<SolutionManager.IO.Solution> solutions = null;

				if (directoryTreeView.SelectedNode != null)
				{
					solutions = (directoryTreeView.SelectedNode.Tag as SolutionManager.IO.SearchDirectory).Solutions;
				}

				_solutionManagerControler.BindSolutions(solutions);
				showExcludedSolutionsToolStripMenuItem.Checked = value;
			}
		}

		public bool ShowDirectoriesView
		{
			get
			{
				return splitContainer.Panel1Collapsed;
			}
			set
			{
				splitContainer.Panel1Collapsed = !value;
				viewSearchDirectoriesToolStripMenuItem.Checked = value;
				showDirectoriesMenuItem.Text = (value) ? "Hide Directories" : "Show Directories";
				AlignListViews();
			}
		}

		public Orientation OrientationLayout
		{
			get
			{
				return splitContainer.Orientation;
			}
			set
			{
				splitContainer.Orientation = value;
			}
		}

		public View SolutionView
		{
			get
			{
				return lstSolutions.View;
			}
			set
			{
				lstSolutions.View = value;
			}
		}

		#endregion

		public bool AllDirectoryTreeNodesExpanded
		{
			get
			{
				return _allDirectoryTreeNodesExpanded;
			}
			set
			{
				if (value)
				{
					directoryTreeView.CollapseAll();
				}
				else
				{
					directoryTreeView.ExpandAll();
				}

				_allDirectoryTreeNodesExpanded = value;
			}
		}

		public SolutionControlControler SolutionManagerControler
		{
			get
			{
				return _solutionManagerControler;
			}
		}

		public DTE2 ApplicationObject
		{
			get
			{
				return _applicationObject;
			}
			set
			{
				_applicationObject = value;
			}
		}

		public EnvDTE.Window ParentWindow
		{
			get
			{
				return _window;
			}
			set
			{
				_window = value;
			}
		}

		#endregion

		#region Methods

		#region Events

		private void SolutionControlView_Load(object sender, EventArgs e)
		{
			_solutionManagerControler = new SolutionControlControler(this, this, this);

			// TODO: LoadSettings
			updateDirectoryToolStripMenuItem.Visible = false;
			solutionManagerSettingsToolStripMenuItem.Visible = false;
			ShowDirectoriesView = true;
		}


		#region ToolStrip Menu

		// First MenuItem
		private void largeIconsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SolutionView = View.LargeIcon;
			largeIconsToolStripMenuItem.Checked = true;
			smallIconsToolStripMenuItem.Checked = false;
			detailsViewToolStripMenuItem.Checked = false;
		}

		private void smallIconsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SolutionView = View.SmallIcon;
			largeIconsToolStripMenuItem.Checked = false;
			smallIconsToolStripMenuItem.Checked = true;
			detailsViewToolStripMenuItem.Checked = false;
		}

		private void detailsViewToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SolutionView = View.Details;
			largeIconsToolStripMenuItem.Checked = false;
			smallIconsToolStripMenuItem.Checked = false;
			detailsViewToolStripMenuItem.Checked = true;
		}

		// Second MenuItem
		private void horizontalLayoutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			horizontalLayoutToolStripMenuItem.Checked = true;
			verticalLayoutToolStripMenuItem.Checked = false;
			OrientationLayout = Orientation.Horizontal;
			AlignListViews();
		}

		private void verticalLayoutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			verticalLayoutToolStripMenuItem.Checked = true;
			horizontalLayoutToolStripMenuItem.Checked = false;
			OrientationLayout = Orientation.Vertical;
			AlignListViews();
		}

		private void showExcludedSolutionsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			ShowExcludedFiles = showExcludedSolutionsToolStripMenuItem.Checked;
		}

		private void solutionManagerSettingsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			//SolutionManagerSettingsForm solutionSettingsForm = new SolutionManagerSettingsForm();
			//solutionSettingsForm.ShowSettingsDialog();
		}

		private void viewSearchDirectoriesToolStripMenuItem_Click(object sender, EventArgs e)
		{
			ShowDirectoriesView = viewSearchDirectoriesToolStripMenuItem.Checked;
		}

		// Third MenuItem
		private void showDirectoriesMenuItem_Click(object sender, EventArgs e)
		{
			if (showDirectoriesMenuItem.Text.Equals("Show Directories"))
			{
				ShowDirectoriesView = true;
			}
			else
			{
				ShowDirectoriesView = false;
			}
		}

		#endregion

		#region SplitContainer

		private void SplitContainer_SplitterMoved(object sender, SplitterEventArgs e)
		{
			AlignListViews();
		}

		#endregion

		#region DirectoryTreeView

		private void directoryTreeView_AfterCollapse(object sender, TreeViewEventArgs e)
		{
			if (!e.Node.Text.Equals("Solution Manager"))
			{
				e.Node.ImageIndex = 1;
			}
		}

		private void directoryTreeView_AfterExpand(object sender, TreeViewEventArgs e)
		{
			if (!e.Node.Text.Equals("Solution Manager"))
			{
				e.Node.ImageIndex = 0;
			}
		}

		private void directoryTreeView_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
		{
			if ((e != null) && (e.Node != null) && (!string.IsNullOrEmpty(e.Label)))
			{
				int nodeCount = directoryTreeView.Nodes.Count;

				if (nodeCount > 0)
				{
					bool rename = true;

					for (int i = 0; i < nodeCount; i++)
					{
						if (e.Label.Equals(directoryTreeView.Nodes[i].Text, StringComparison.Ordinal))
						{
							rename = false;
							break;
						}
					}

					if (rename)
					{
						_solutionManagerControler.RenameSearchDirectory(e.Node.Text, e.Label);
					}
					else
					{
						e.CancelEdit = true;
					}
				}
			}
		}

		private void directoryTreeView_BeforeLabelEdit(object sender, NodeLabelEditEventArgs e)
		{
			if (e.Node.Text.Equals("Solution Manager"))
			{
				e.CancelEdit = true;
			}
		}

		private void directoryTreeView_DragDrop(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent("System.Windows.Forms.TreeNode", true))
			{
				TreeNode dragNode = null;

				System.Drawing.Point point = new System.Drawing.Point(e.X, e.Y);
				point = ((TreeView)sender).PointToClient(point);

				TreeNode nodeToAddTo = ((TreeView)sender).GetNodeAt(point);
				dragNode = (TreeNode)e.Data.GetData("System.Windows.Forms.TreeNode");

				if (nodeToAddTo.TreeView == dragNode.TreeView)
				{
					if (nodeToAddTo != dragNode)
					{
						nodeToAddTo.Nodes.Add((TreeNode)dragNode.Clone());
						nodeToAddTo.Expand();

						_solutionManagerControler.MoveDirectoryNode(dragNode, dragNode.Parent, nodeToAddTo);

						dragNode.Remove();
					}
				}
			}
		}

		private void directoryTreeView_DragEnter(object sender, DragEventArgs e)
		{
			e.Effect = DragDropEffects.All;
		}

		private void directoryTreeView_ItemDrag(object sender, ItemDragEventArgs e)
		{
			directoryTreeView.DoDragDrop(e.Item, DragDropEffects.All);
		}

		private void directoryTreeView_MouseClick(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Right)
			{
				// Show the ExpandAll or CollapsAll menu items
				if (_allDirectoryTreeNodesExpanded)
				{
					expandAllToolStripMenuItem.Visible = true;
					collapsAllToolStripMenuItem.Visible = false;
				}
				else
				{
					collapsAllToolStripMenuItem.Visible = true;
					expandAllToolStripMenuItem.Visible = false;
				}
			}

			// Enable or disable the DirectoryContextMenu buttons if no nodes are selected.
			EnableDesableDirectoryContextMenuButtons((directoryTreeView.SelectedNode != null));
		}

		private void directoryTreeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
		{
			SetWindowText(e.Node.Text, e.Node.ToolTipText);
			SearchDirectory directory = e.Node.Tag as SearchDirectory;

			if (directory != null)
			{
				if (directory.Solutions != null)
				{
					_solutionManagerControler.BindSolutions(directory.Solutions);
				}
			}

			directoryTreeView.SelectedNode = e.Node;
		}

		#endregion

		#region DirectoryContextMenu

		private void collapsAllToolStripMenuItem_Click(object sender, EventArgs e)
		{
			AllDirectoryTreeNodesExpanded = true;
		}

		private void deleteToolStripMenuItem1_Click(object sender, EventArgs e)
		{
			if (directoryTreeView.SelectedNode != null)
			{
				DialogResult result = MessageBox.Show(string.Format("Are you sure you want to exclude the following items?\n{0}", directoryTreeView.SelectedNode.Text), "Warning!", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);

				if (result == DialogResult.Yes)
				{
					if (directoryTreeView.SelectedNode != null)
					{
						string parentName = "Solution Manager";

						if (directoryTreeView.SelectedNode.Parent != null)
						{
							parentName = directoryTreeView.SelectedNode.Parent.Text;
						}

						_solutionManagerControler.RemoveDirectoryNode(directoryTreeView.SelectedNode, parentName);
					}
				}
			}
		}

		private void expandAllToolStripMenuItem_Click(object sender, EventArgs e)
		{
			AllDirectoryTreeNodesExpanded = false;
		}

		private void minimizeToolStripMenuItem_Click(object sender, EventArgs e)
		{
			ShowDirectoriesView = false;
		}

		private void newFolderToolStripMenuItem_Click(object sender, EventArgs e)
		{
			AddSearchDirectory();
		}

		private void openToolStripMenuItem1_Click(object sender, EventArgs e)
		{
			string directory = null;

			if (directoryTreeView.SelectedNode != null)
			{
				directory = directoryTreeView.SelectedNode.ToolTipText;
				OpenContainingFolder(directory, string.Empty);
			}
		}

		private void updateDirectoryToolStripMenuItem_Click(object sender, EventArgs e)
		{
		}

		#endregion

		#region lstSolutions

		private void lstSolutions_Click(object sender, EventArgs e)
		{
			int selectedItemsCount = lstSolutions.SelectedItems.Count;

			if (selectedItemsCount > 0)
			{
				for (int i = 0; i < selectedItemsCount; i++)
				{
					if (!System.IO.File.Exists(lstSolutions.SelectedItems[i].SubItems[1].Text))
					{
						MessageBox.Show(string.Format("The following Solution no longer exists and will be removed from the list.\n\"{0}\"", lstSolutions.SelectedItems[i].Text));
						_solutionManagerControler.ExcludeSolutionNode(lstSolutions.SelectedItems[i], (i == selectedItemsCount - 1));
					}
				}
			}
		}

		private void lstSolutions_DoubleClick(object sender, EventArgs e)
		{
			OpenSolution();
		}

		#endregion

		#region lstSolutionsContextMenu

		private void openLstSolutionsMenuItem_Click(object sender, EventArgs e)
		{
			OpenSolution();
		}

		private void openToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if (lstSolutions.SelectedItems.Count > 0)
			{
				System.Diagnostics.Process.Start(lstSolutions.SelectedItems[0].SubItems[1].Text);
			}
		}

		private void openContainingFolderToolStripMenuItem_Click(object sender, EventArgs e)
		{
			int itemCount = lstSolutions.SelectedItems.Count;

			if (itemCount > 0)
			{
				for (int i = 0; i < itemCount; i++)
				{
					OpenContainingFolder(lstSolutions.SelectedItems[i].SubItems[1].Text, lstSolutions.SelectedItems[i].Text);
				}
			}
		}

		private void removeToolStripMenuItem_Click(object sender, EventArgs e)
		{
			int selectedItemsCount = lstSolutions.SelectedItems.Count;

			if (selectedItemsCount > 0)
			{
				DialogResult result = MessageBox.Show("Are you sure you want to delete the selected items?\n", "Warning!", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);

				if (result == DialogResult.Yes)
				{
					ListView.SelectedListViewItemCollection items = lstSolutions.SelectedItems;

					for (int i = 0; i < selectedItemsCount; i++)
					{
						_solutionManagerControler.ExcludeSolutionNode(items[i], (i == selectedItemsCount - 1));
					}
				}
			}
		}

		#endregion

		#endregion

		#region Helper Methods

		private void AddSearchDirectory()
		{
			if (directoryTreeView.SelectedNode != null)
			{
				SolutionManager.Business.UI.Froms.NewFolderForm newFolderForm = new SolutionManager.Business.UI.Froms.NewFolderForm(directoryTreeView.SelectedNode);
				
				// Need to change: newFolderForm.NewFolderShowDialog(false);
				SolutionManager.Business.UI.Froms.IDirectorySettings directorySettings = newFolderForm.NewFolderShowDialog(true);

				if (directorySettings != null)
				{
					if (directorySettings.AddSearch)
					{
						_solutionManagerControler.DisplayMessageInSolutionsView("Searching...");
						System.Threading.ThreadPool.QueueUserWorkItem(_solutionManagerControler.AddSearchDirectory, directorySettings);
					}
					else
					{
						directoryTreeView.SelectedNode.Nodes.Add(directorySettings.Path, directorySettings.FriendlyName, 0);
						_solutionManagerControler.AddVirtualNode(directorySettings);
					}
				}
			}
		}

		private void AlignListViews()
		{
			if ((lstSolutions != null) && (lstSolutions.Columns.Count > 0))
			{
				lstSolutions.Columns[1].Width = ((splitContainer.Panel2.Width - lstSolutions.Columns[0].Width) - (lstSolutions.Columns[2].Width + 25));
			}
		}

		private void EnableDesableDirectoryContextMenuButtons(bool isEnabled)
		{
			newFolderToolStripMenuItem.Enabled = isEnabled;
			deleteToolStripMenuItem1.Enabled = isEnabled;
			openToolStripMenuItem1.Enabled = isEnabled;
			updateDirectoryToolStripMenuItem.Enabled = isEnabled;
		}

		private void OpenSolution()
		{
			if (lstSolutions.SelectedItems.Count > 0)
			{
				try
				{
					if (_applicationObject.Solution != null)
					{
						_applicationObject.Solution.Open(lstSolutions.SelectedItems[0].SubItems[1].Text);
					}
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.Message);
				}
			}
		}

		public void OpenContainingFolder(string directoryName, string fileName)
		{
			if ((!string.IsNullOrEmpty(directoryName)))
			{
				if (!string.IsNullOrEmpty(fileName))
				{
					string directory = directoryName.Replace(fileName, string.Empty);
					System.Diagnostics.Process.Start(@"explorer.exe", directory);
				}
				else
				{
					System.Diagnostics.Process.Start(@"explorer.exe", directoryName);
				}
			}
		}

		private void SetWindowText(string title1, string title2)
		{
			if (!title1.Equals("Solution Manager"))
			{
				_window.Caption = string.Format(_SolutionManagerTitle, " -- ", title1, title2);
			}
			else
			{
				_window.Caption = string.Format(_SolutionManagerTitle, string.Empty, string.Empty, string.Empty);
			}
		}

		public object SolutionControlViewInvoke(Delegate del)
		{
			return Invoke(del);
		}

		#endregion

		#endregion
	}
}